package Apache::AutoSession;

use strict;
use Apache::Constants qw(:common);
use Carp;

sub DEBUG { 0 }

##############################################
# Main handler, punts the activity to _handler
sub handler 
{
    my($r)=@_;

    DEBUG and $r->warn(__PACKAGE__."::handler");

    # find the session directory to use
    my $confname=$r->dir_config('AutoSessionDir');
    unless($confname)
    {
        $r->warn("AutoSessionDir isn't specified");
        return Apache::Constants::DECLINED;
    }
    my $lockname=$r->dir_config('AutoSessionLockDir');
    $lockname||=$confname;

    my $var=$r->dir_config("AutoSession") || '%'.__PACKAGE__.'session';
    my $sub=$r->dir_config("AutoSessionSub") || 
                        sub { __PACKAGE__->load(@_, $lockdir)};

    return Apache::AutoConfig::_load($r, $confname, $var, '', $sub);
}



################################################################
sub load {
    my($r, $confdir, $glob, $type, $lockdir)=@_;
    my $id=_get_id($r);

    DEBUG and $r->warn("Tying $confdir/$id to $glob");
    if(tie %$glob, 'Apache::AutoSession::File', $id, 
       {Directory => $confdir, LockDir=>$lockdir, LockDirectory => $lockdir}) {
        $r->register_cleanup(sub 
            {
                my $foo=1 if(defined %$glob and tied %$glob);
                DEBUG and $r->warn(__PACKAGE__." cleanup $$", $foo ? '' : ' already untied');
                untie %$glob if $foo
            });
        return 1;
    }
    return 0;
}

################################################################
sub _get_id {
    my($r)=@_;
    my $id_='ID';                               # TODO
    my $n=$r->notes($id_) ;
    return $n if $n;

    return sprintf "%x%x%x", time, $$, rand();
}

################################################################
package Apache::AutoSession::File;

use vars qw(@ISA);
@ISA = qw(Apache::Session);

use Apache::Session;
BEGIN {
    if($Apache::Session::VERSION > 1) {
        require Apache::Sesssion::File;
        @ISA=qw(Apache::Session::File);
    } else {
        warn "Using old version of Apache::Session";
        require Apache::Session::PosixFileLocker;
        require Apache::Session::FileStore;
    }
}

}
sub get_object_store {
    my $self = shift;

    return new Apache::Session::FileStore $self;
}

sub get_lock_manager {
    my $self = shift;

    return new Apache::Session::PosixFileLocker $self;
}


1;

__END__

=head1 NAME

Apache::AutoSession - Automatically load an Apache::Session

=head1 SYNOPSIS

  package My::Code;
  use Apache::AutoSession;
  use vars qw(%session);

  sub handler
  {
     my($r)=@_;
     # use %session with impunity, it will be untied when the 
     # request is finished
  }

And in an .htaccess:

  <IfModule mod_perl.c>
    <FilesMatch  "tellabout.+\.html$">
      PerlInitHandler Apache::AutoSession Apache::AutoID
      PerlSetVar AutoSession %My::Code::session
      PerlSetVar AutoSessionDir /tmp/sessions
    </FilesMatch>    
  </IfModule>

=head1 DESCRIPTION

Apache::AutoSession is used to bind Apache::Session and Apache::AutoID.

=head2 PerlSetVars

=over 4

=item AutoSession

The variable that should be tied to Apache::Session.

=item AutoSessionDir

The directory that contains the session files.

=item AutoSessionSub

A subroutine that will be called to do the tying.  Use this if you want to
do something other then tie to Apache::AutoSession::File.

    sub mytie 
    {
        my($r, $confdir, $glob)=@_;
        #
        ## Some how we defined $id to the id...
        tie %$glob, 'Apache::DataSession::DBI', $id, 
            {DataSource => 'dbi:Oracle:db'};
        ...
    }

=back

The default behaviour is to tie AutoSession to L<Apache::AutoSession::File>,
which uses L<Apache::Session::FileStore>, L<Apache::Session::PosixFileLocker>. 
If you wish to tie to database, define your own AutoSessionSub.

This module requires L<Apache::AutoID> or something else to put an 'ID' in
$r->notes().  If there is no notes, it generates a new ID.  use
AutoSessionSub to be able to pull the session ID from elsewhere.

=head1 BUGS

See L<Apache::AutoID/BUGS>.

=head1 TODO

-Use Memoize to cache tied values.  This would alleviate the need to reread
the config file each and every access.

=head1 AUTHOR

Philip Gwyn, <fil@pied.nu>

=head1 SEE ALSO

Apache::AutoSession, Apache::AutoID, perl(1).

=cut
